"""
Module: constant.py
Description: This module contains the constant used in airstack sdk.
"""

class AirstackConstants:
    """Class for keeping constants
    """
    API_ENDPOINT_PROD = 'https://api.airstack.xyz/gql'
    API_TIMEOUT = 60
    SUCCESS_STATUS_CODE = 200
    UNPROCESSABLE_STATUS_CODE = 422
